// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDirectDrawClipper.CPP
// Description:     This file contains the code for the TDirectDrawClipper
//                  component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// Include the TDirectDrawClipper H File
// --------------------------------------------------------------------------
#include "TDirectDrawClipper.h"

// --------------------------------------------------------------------------
// Method:          Constructor
// Description:     Sets the default values used within TDirectDrawClipper
//                  and creates any miscellaneous internal objects
// --------------------------------------------------------------------------
                    __fastcall TDirectDrawClipper::TDirectDrawClipper
                                    (TComponent* Owner) : TComponent(Owner)
{
fError = DD_OK;
}

// --------------------------------------------------------------------------
// Method:          Destructor
// Description:     Release / delete anything created in the constructor
// --------------------------------------------------------------------------
                    __fastcall  TDirectDrawClipper::~TDirectDrawClipper()
{
}

// --------------------------------------------------------------------------
// Method:          GetClipList
// Description:     Retrieves a copy of the clip list associated with a
//                  DirectDrawClipper object
// Parameters:      ClipList    - Address of an RGNDATA structure that will
//                              contain the resulting copy of the clip list
//                              If this parameter is NULL, the method fills
//                              the variable at lpdwSize to the number of
//                              bytes necessary to hold the entire clip list
//                  Rect        - Address of a rectangle that will be used to
//                              clip the clip list. This parameter can be
//                              NULL to retrieve the entire clip list
//                  Size        - Size of the resulting clip list. When
//                              retrieving the clip list, this parameter is
//                              the size of the buffer at ClipList. When
//                              ClipList is NULL, the variable at lpdwSize
//                              receives the required size of the buffer, in
//                              bytes
// Note:            A subset of the clip list can be selected by passing a
//                  rectangle that clips the clip list
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawClipper::GetClipList()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawClipper::GetClipList()",
            "TDX_NOTCREATED",
                "The TDirectDrawClipper interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawClipper::GetClipList()",
                                        "about to retrieve a clip list");
// everything worked as expected
FOnMessage(this,"TDirectDrawClipper::GetClipList()",
                                "the clip list was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetClipList
// Description:     Sets or deletes the clip list used by the Blt, BltBatch,
//                  and UpdateOverlay methods on surfaces to which the parent
//                  Clipper object is attached
// Parameters:      ClipList    - Either an address to a valid RGNDATA
//                  structure or NULL. If there is an existing clip list
//                  associated with the DirectDrawClipper object and this
//                  value is NULL, the clip list will be deleted
//                  Flags       -  This parameter is currently not used and
//                              must be set to 0
// Note:            The clip list cannot be set if a window handle is already
//                  associated with the Clipper object
//                  The BltFast method cannot clip
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawClipper::SetClipList()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawClipper::SetClipList()",
            "TDX_NOTCREATED",
                "The TDirectDrawClipper interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawClipper::SetClipList()",
                                        "about to set a clip list");
// everything worked as expected
FOnMessage(this,"TDirectDrawClipper::SetClipList()",
                                "the clip list was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetHWnd
// Description:     Retrieves the window handle previously associated with
//                  this DirectDrawClipper object by the SetHWnd method
// Parameters:      hWnd    - Address of the window handle previously
//                          associated with this Clipper object by the
//                          SetHWnd method
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawClipper::GetHWnd()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawClipper::GetHWnd()","TDX_NOTCREATED"
                ,"The TDirectDrawClipper interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawClipper::GetHWnd()",
                                    "about to retrieve the window handle");
// everything worked as expected
FOnMessage(this,"TDirectDrawClipper::GetHWnd()",
                                "the window handle was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetHWnd
// Description:     Sets the window handle that will obtain the clipping
//                  information
// Parameters:      Flags   - This parameter is currently not used and must
//                          be set to 0
//                  hWnd    - Window handle that obtains the clipping
//                          information
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawClipper::SetHWnd()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawClipper::SetHWnd()",
            "TDX_NOTCREATED",
                "The TDirectDrawClipper interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawClipper::SetHWnd()",
                                        "about to set the window handle");
// everything worked as expected
FOnMessage(this,"TDirectDrawClipper::Blt()",
                                    "the window handle was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Initialize
// Description:     Initializes a Clipper object that was created by using
//                  the CoCreateInstance COM function
// Parameters:      DD      - Address of the DirectDraw structure that
//                          represents the DirectDraw object. If this
//                          parameter is set to NULL, an independent
//                          DirectDrawClipper object is created
//                  Flags   - This parameter is currently not used and must
//                          be set to 0
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawClipper::Initialize()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawClipper::Initialize()",
            "TDX_NOTCREATED",
                "The TDirectDrawClipper interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawClipper::Initialize()", "about to initialize");
//DJD
//fError = fSurfacePtr->Blt();
// everything worked as expected
FOnMessage(this,"TDirectDrawClipper::Initialize()",
                                        "the initialization was successful");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          IsClipListChanged
// Description:     Monitors the status of the clip list if a window handle
//                  is associated with a Clipper object
// Parameters:      Changed - Address of a variable that is set to TRUE if
//                          the clip list has changed
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawClipper::IsClipListChanged()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawClipper::IsClipListChanged()",
            "TDX_NOTCREATED",
                "The TDirectDrawClipper interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawClipper::IsClipListChanged()",
                                            "about to check the clip list");
// everything worked as expected
FOnMessage(this,"TDirectDrawClipper::IsClipListChanged()",
                                    "the clip list was checked correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------

