// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDirectDrawPalette.CPP
// Description:     This file contains the code for the TDirectDrawPalette
//                  component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// Include the TDirectDrawPalette H File
// --------------------------------------------------------------------------
#include "TDirectDrawPalette.h"

// --------------------------------------------------------------------------
// Method:          Constructor
// Description:     Sets the default values used within TDirectDrawClipper
//                  and creates any miscellaneous internal objects
// --------------------------------------------------------------------------
                    __fastcall TDirectDrawPalette::TDirectDrawPalette
                                    (TComponent* Owner) : TComponent(Owner)
{
fError = DD_OK;
}

// --------------------------------------------------------------------------
// Method:          Destructor
// Description:     Release / delete anything created in the constructor
// --------------------------------------------------------------------------
                    __fastcall  TDirectDrawPalette::~TDirectDrawPalette()
{
}



// --------------------------------------------------------------------------
// Method:          GetCaps
// Description:     Retrieves the capabilities of this palette object
// Parameters:      Caps    - Flag from the dwPalCaps member of the DDCAPS
//                          structure that defines palette capabilities
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawPalette::GetCaps()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawPalette::GetCaps()",
            "TDX_NOTCREATED",
                "The TDirectDrawPalette interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawPalette::GetCaps()",
                                        "about to retrieve capabilities");
// everything worked as expected
FOnMessage(this,"TDirectDrawPalette::GetCaps()",
                                "the capabilities were retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetEntries
// Description:     Queries palette values from a DirectDrawPalette object
// Parameters:      Base        - Start of the entries that should be
//                              retrieved sequentially
//                  Entries     - Address of the palette entries. Each field
//                              is a color description
//                  Flags       - This parameter is currently not used and
//                              must be set to 0
//                  NumEntries  - Number of palette entries that can fit in
//                              the address specified in Entries. The
//                              colors of each palette entry are returned in
//                              sequence, from the value of the StartingEntry
//                              parameter through the value of the Count
//                              parameter minus 1 (These parameters are set
//                              by the SetEntries method)
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawPalette::GetEntries()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawPalette::GetEntries()",
            "TDX_NOTCREATED",
                "The TDirectDrawPalette interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawPalette::GetEntries()",
                                        "about to retrieve palette entries");
// everything worked as expected
FOnMessage(this,"TDirectDrawPalette::GetEntries()",
                            "the palette entries were retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetEntries
// Description:     Changes entries in a Palette object immediately
// Parameters:      Count           - Number of palette entries to be changed
//                  Entries         - Address of the palette entries. Each
//                                  field is a color description
//                  Flags           - This parameter is currently not used
//                                  and must be set to 0
//                  StartingEntry   - First entry to be set
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawPalette::SetEntries()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawPalette::SetEntries()",
            "TDX_NOTCREATED",
                "The TDirectDrawPalette interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawPalette::SetEntries()",
                                        "about to set palette entries");
// everything worked as expected
FOnMessage(this,"TDirectDrawPalette::SetEntries()",
                                "the palette entries were set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Initialize
// Description:     Initializes the TDirectDrawPalette object
// Parameters:      ColorTable  - This parameter is currently not used and
//                              must be set to 0
//                  DD          - Address of the DirectDraw structure that
//                              represents the DirectDraw object
//                  Flags       - This parameter is currently not used and
//                              must be set to 0
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawPalette::Initialize()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawPalette::Initialize()",
            "TDX_NOTCREATED",
                "The TDirectDrawPalette interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawPalette::Initialize()",
                                        "about to intialize palette object");
// everything worked as expected
FOnMessage(this,"TDirectDrawPalette::Initialize()",
                            "the palette object was initialized correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
