// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDirectDrawVideoPort.CPP
// Description:     This file contains the code for the TDirectDrawVideoPort
//                  component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// Include the TDirectDrawClipper H File
// --------------------------------------------------------------------------
#include "TDirectDrawVideoPort.h"

// --------------------------------------------------------------------------
// Method:          Constructor
// Description:     Sets the default values used within TDirectDrawVideoPort
//                  and creates any miscellaneous internal objects
// --------------------------------------------------------------------------
                    __fastcall TDirectDrawVideoPort::TDirectDrawVideoPort
                                    (TComponent* Owner) : TComponent(Owner)
{
fError = DD_OK;
}

// --------------------------------------------------------------------------
// Method:          Destructor
// Description:     Release / delete anything created in the constructor
// --------------------------------------------------------------------------
                    __fastcall  TDirectDrawVideoPort::~TDirectDrawVideoPort()
{
}

// --------------------------------------------------------------------------
// Method:          Flip
// Description:     Instructs the VideoPort object to write the next frame of
//                  video to a new surface
// Parameters:      Flags   - Flip options flags
//                  Surface - Address of the DirectDrawSurface object that
//                          will receive the next frame of video
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::Flip()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::Flip()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::Flip()",
                                        "about to write the next frame");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::Flip()",
                                "the frame was written correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetBandwidthInfo
// Description:     Retrieves the minimum required overlay zoom factors and
//                  device limitations of a video port that uses the provided
//                  output pixel format
// Parameters:      Bandwidth   - Address of a DDVIDEOPORTBANDWIDTH structure
//                              that will be filled with the retrieved
//                              bandwidth and device dependency information
//                  Flags       - Indicates how the method is to interpret
//                              the Width and Height parameters
//                  Format      - Address of a DDPIXELFORMAT structure that
//                              describes the output pixel format for which
//                              bandwidth information will be retrieved
//                  Height      - Vertical dimension of an overlay or video
//                              data. The interpretation of this parameter
//                              depends on the value specified in the dwFlags
//                              parameter
//                  Width       - Horizontal dimension of an overlay or video
//                              data. The interpretation of this parameter
//                              depends on the value specified in the Flags
//                              parameter
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetBandwidthInfo()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::GetBandwidthInfo()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetBandwidthInfo()",
                                        "about to retrieve bandwidth info");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetBandwidthInfo()",
                            "the bandwidth info was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetColorControls
// Description:     Returns the current color control settings associated
//                  with the video port
// Parameters:      ColorControl    - Address of a DDCOLORCONTROL structure
//                                  that will be filled with the current
//                                  settings of the video port's color
//                                  control
// Note:            The Flags member of the DDCOLORCONTROL structure indicate
//                  which of the color control options are supported
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetColorControls()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::GetColorControls()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetColorControls()",
                                    "about to retrieve the color controls");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetColorControls()",
                            "the color controls were retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetFieldPolarity
// Description:     Retrieves the polarity of a video field
// Parameters:      FieldPolarity   - Address of a variable that will be set
//                                  to indicate the current field polarity.
//                                  This value is set to true if the current
//                                  video field is the even field of an
//                                  interlaced video signal and false
//                                  otherwise
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetFieldPolarity()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::GetFieldPolarity()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetFieldPolarity()",
                                        "about to retrieve field polarity");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetFieldPolarity()",
                            "the field polarity was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetInputFormats
// Description:     Retrieves the input formats supported by the VideoPort
//                  object
// Parameters:      Flags       - Specifies the part of the video signal for
//                              which formats will be enumerated
//                  Formats     - Address of an array of DDPIXELFORMAT
//                              structures that will be filled in with the
//                              input formats supported by this VideoPort
//                              object. If this parameter is NULL, the method
//                              sets NumFormats to the number of supported
//                              formats and the returns DD_OK
//                  NumFormats  - Address of a variable containing the number
//                              of entries that the array at Formats can hold
//                              If this number is less than the total number
//                              of codes, the method fills the array with as
//                              many codes as will fit, sets the value at
//                              NumFormats to indicate the total number of
//                              codes, and returns DDERR_MOREDATA
// Note:            This method can also be used to return the number of
//                  formats supported. To do this, set the Formats parameter
//                  to NULL. When the method returns, the variable at
//                  NumFormats contains the total number of supported input
//                  formats
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetInputFormats()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::GetInputFormats()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetInputFormats()",
                                        "about to retrieve input formats");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetInputFormats()",
                            "the input formats were retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetOutPutFormats
// Description:     Retrieves a list of output formats that the VideoPort
//                  object supports for a specified input format
// Parameters:      Flags       - Specifies the part of the video signal for
//                              which formats will be enumerated
//                  Formats     - Address of an array of DDPIXELFORMAT
//                              structures that will be filled in with the
//                              output formats supported by this VideoPort
//                              object. If this parameter is NULL, the method
//                              sets NumFormats to the number of supported
///                             formats and the returns DD_OK
//                  InputFormat - Address of a DDPIXELFORMAT structure that
//                              describes the input format for which
//                              conversion information is requested
//                  NumFormats  - Address of a variable containing the number
//                              of entries that the array at Formats can hold
//                              If this number is less than the total number
//                              of codes, the method fills the array with as
//                              many codes as will fit, sets the value at
//                              NumFormats to indicate the total number of
//                              codes, and returns DDERR_MOREDATA
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetOutputFormats()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::GetOutputFormats()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetOutputFormats()",
                                    "about to retrieve the output formats");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetOutputFormats()",
                            "the output formats were retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetVideoLine
// Description:     Retrieves the current line of video being written to the
//                  frame buffer
// Parameters:      Line    - Address of a variable that will be filled with
//                          a value indicating the video line currently being
//                          written to the frame buffer
// Note:            The value this method retrieves reflects the true video
//                  line being written, relative to the field height, before
//                  any prescaling occurs
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetVideoLine()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::GetVideoLine()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetVideoLine()",
                                        "about to retrieve a video line");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetVideoLine()",
                                "the video line was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetVideoSignalStatus
// Description:     Retrieves the status of the video signal currently being
//                  presented to the video port
// Parameters:      Status  - Address of a variable that will contain a
//                          return code indicating the quality of the video
//                          signal at the video port
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::GetVideoSignalStatus()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,
            "TDirectDrawVideoPort::GetVideoSignalStatus()", "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::GetVideoSignalStatus()",
                            "about to retrieve the video signal status");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::GetVideoSignalStatus()",
                        "the video signal status was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetColorControls
// Description:     Sets the color control settings associated with the video
//                  port
// Parameters:      ColorControl    - Address of a DDCOLORCONTROL structure
//                                  containing the new color control settings
//                                  that will be applied to the video port
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::SetColorControls()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::SetColorControls()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::SetColorControls()",
                                        "about to set the color controls");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::SetColorControls()",
                                "the color controls were set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetTargetSurface
// Description:     Sets the DirectDraw surface object that will receive the
//                  stream of live video data and/or the vertical blank
//                  interval data
// Parameters:      Flags   - Value specifying the type of target surface
//                  Surface - Address of the Surface object that will receive
//                          the video data
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::SetTargetSurface()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::SetTargetSurface()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::SetTargetSurface()",
                                        "about to set the target surface");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::SetTargetSurface()",
                                "the target surface was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          StartVideo
// Description:     Enables the hardware video port and starts the flow of
//                  video data into the currently specified surface
// Parameters:      VideoInfo   - Address of a pointer to a DDVIDEOPORTINFO
//                              structure
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::StartVideo()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::StartVideo()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::StartVideo()",
                                        "about to start the video stream");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::StartVideo()",
                                "the video stream was started correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          StopVideo
// Description:     Stops the flow of video port data into the frame buffer
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::StopVideo()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::StopVideo()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::StopVideo()",
                                                "about to stop the video");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::StopVideo()",
                                        "the video was stopped correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          UpdateVideo
// Description:     Updates parameters that govern the flow of video data
//                  from the video-port to the Surface object
// Parameters:      VideoInfo   -   Address of a DDVIDEOPORTINFO structure
//                              that describes the video transfer parameters
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::UpdateVideo()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::UpdateVideo()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::UpdateVideo()",
                                        "about to update video parameters");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::UpdateVideo()",
                            "the video parameters were updated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          WaitForSync
// Description:     Waits for VSYNC or until a given scan line is being drawn
// Parameters:      Flags   - Specifies how the method will wait for the
//                          video VSYNC or the specified line number
//                  Line    - The video line determining when the method
//                          should return, relative to the field height,
//                          before prescaling. This parameter is ignored if
//                          the Flags parameter is set to
//                          DDVPWAIT_BEGINVBLANK or DDVPWAIT_ENDVBLANK
//                  Timeout - Amount of time, in milliseconds, that the
//                          method will wait for the next video vertical
//                          blank before timing out. If this parameter is 0,
//                          the method waits 3 times the value specified in
//                          the MicrosecondsPerField member of the
//                          DDVIDEOPORTDESC
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::WaitForSync()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawVideoPort::WaitForSync()",
            "TDX_NOTCREATED",
                "The TDirectDrawVideoPort interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPort::WaitForSync()",
                                    "about to wait for sync or scan line");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPort::WaitForSync()",
                                "the wait was performed correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

