// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDirectDrawVideoPortContainer.CPP
// Description:     This file contains the code for the
//                  TDirectDrawVideoPortContainer component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// Include the TDirectDrawClipper H File
// --------------------------------------------------------------------------
#include "TDirectDrawVideoPortContainer.h"

// --------------------------------------------------------------------------
// Method:          Constructor
// Description:     Sets the default values used within TDirectDrawVideoPort
//                  and creates any miscellaneous internal objects
// --------------------------------------------------------------------------
                    __fastcall
                TDirectDrawVideoPortContainer::TDirectDrawVideoPortContainer
                                    (TComponent* Owner) : TComponent(Owner)
{
fError = DD_OK;
}

// --------------------------------------------------------------------------
// Method:          Destructor
// Description:     Release / delete anything created in the constructor
// --------------------------------------------------------------------------
                    __fastcall
            TDirectDrawVideoPortContainer::~TDirectDrawVideoPortContainer()
{
}

// --------------------------------------------------------------------------
// Method:          CreateVideoPort
// Description:     Creates a VideoPort object
// Parameters:      DDVideoPort     - Address of a variable that will be
//                                  filled with a pointer to a new VideoPort
//                                  object's IDirectDrawVideoPort interface
//                                  if the call succeeds
//                  DDVideoPortDesc - Address of a DDVIDEOPORTDESC structure
//                                  that describes the VideoPort object to be
//                                  created
//                  Flags           - Flags specifying video-port control
//                                  options
//                  UnkOuter        - Allows for future compatibility with
//                                  COM aggregation features. Presently,
//                                  however, this method will return an error
//                                  if this parameter is anything but NULL
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::CreateVideoPort()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,
        "TDirectDrawVideoPortContainer::CreateVideoPort()", "TDX_NOTCREATED",
        "The TDirectDrawVideoPortContainer interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPortContainer::CreateVideoPort()",
                                        "about to create the video port");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPortContainer::CreateVideoPort()",
                                "the video port was created correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          EnumVideoPorts
// Description:     Enumerates all of the video ports that the hardware
//                  exposes that are compatible with a provided video port
//                  description
// Parameters:      Context             - Address of a caller-defined
//                                      structure that will be passed to each
//                                      enumeration member
//                  DDVideoPortCaps     - Pointer to a DDVIDEOPORTCAPS
//                                      structure that will be checked
//                                      against the available video ports. If
//                                      this parameter is NULL, all video
//                                      ports will be enumerated
//                  EnumVideoCallback   - Address of the EnumVideoCallback
//                                      function that will be called each
//                                      time a match is found
//                  Flags               - Reserved for future use. This
//                                      parameter must be zero
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::EnumVideoPorts()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,
        "TDirectDrawVideoPortContainer::EnumVideoPorts()", "TDX_NOTCREATED",
        "The TDirectDrawVideoPortContainer interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPortContainer::EnumVideoPorts()",
                                        "about to enum the video ports");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPortContainer::EnumVideoPorts()",
                                "the video ports were enum'd correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetVideoPortConnectInfo
// Description:     Retrieves the connection information supported by all
//                  video ports
// Parameters:      ConnectInfo - Address of an array of DDVIDEOPORTCONNECT
//                              structures that will be filled with the
//                              connection options supported by the specified
//                              video port. If this parameter is NULL, the
//                              method sets lpNumEntries to indicate the
//                              total number of connections that the video
//                              port supports, then returns DD_OK
//                  NumEntries  - Address of a variable containing the number
//                              of entries that the array at ConnectInfo can
//                              hold. If this number is less than the total
//                              number of connections, the method fills the
//                              array with as many entries as will fit, sets
//                              the value at NumEntries to indicate the total
//                              number of connections, and returns
//                              DDERR_MOREDATA
//                  PortId      - Video port ID of the video port for which
//                              the connection information will be retrieved
// --------------------------------------------------------------------------
HRESULT             __fastcall
                            TDirectDrawVideoPort::GetVideoPortConnectInfo()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,
        "TDirectDrawVideoPortContainer::GetVideoPortConnectInfo()",
        "TDX_NOTCREATED",
        "The TDirectDrawVideoPortContainer interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPortContainer::GetVideoPortConnectInfo()",
                            "about to get the video port connection info");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPortContainer::GetVideoPortConnectInfo()",
                "the video port connection info was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          QueryVideoPortStatus
// Description:
// Parameters:      PortId      - Video port ID of the video port for which
//                              the status information will be retrieved
//                  VPStatus    - Address of a DDVIDEOPORTSTATUS structure
//                              that will be filled with information about
//                              the status of the specified video port
// Note:            Supported in Windows 98 and Windows NT 5.0 only
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDrawVideoPort::QueryVideoPortStatus()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,
    "TDirectDrawVideoPortContainer::QueryVideoPortStatus()","TDX_NOTCREATED",
        "The TDirectDrawVideoPortContainer interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawVideoPortContainer::QueryVideoPortStatus()",
                            "about to query the status of the video port");
// everything worked as expected
FOnMessage(this,"TDirectDrawVideoPortContainer::QueryVideoPortStatus()",
                            "the video port status was queried correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

